
/**
 * @imports
 */
import _beforeLast from '@web-native-js/commons/str/beforeLast.js';
import _arrFrom from '@web-native-js/commons/arr/from.js';
import _each from '@web-native-js/commons/obj/each.js';
// -------------------------------------
import el from './dom/el.js';
import itemize from './dom/itemize.js';
import select from './dom/select.js';
import selectAll from './dom/selectAll.js';
import data from './dom/data.js';
// -------------------------------------
import attrAsync from './dom/attrAsync.js';
import attrSync from './dom/attrSync.js';
import classAsync from './dom/classAsync.js';
import classSync from './dom/classSync.js';
import htmlAsync from './dom/htmlAsync.js';
import htmlSync from './dom/htmlSync.js';
import textAsync from './dom/textAsync.js';
import textSync from './dom/textSync.js';
import appendAsync from './dom/appendAsync.js';
import appendSync from './dom/appendSync.js';
import prependAsync from './dom/prependAsync.js';
import prependSync from './dom/prependSync.js';
import mutationCallback from './dom/mutationCallback.js';
import connectedCallback from './dom/connectedCallback.js';
import disconnectedCallback from './dom/disconnectedCallback.js';
import attrChangeCallback from './dom/attrChangeCallback.js';
// -------------------------------------
import cssSync from './css/cssSync.js';
import cssAsync from './css/cssAsync.js';
import cssInline from './css/readInline.js';
import cssRendering from './css/readRendering.js';
import cssStylesheet from './css/readStylesheet.js';
import cssTransaction from './css/transaction.js';
import xBorders from './css/xBorders.js';
import yBorders from './css/yBorders.js';
import zHeight from './css/zHeight.js';
// -------------------------------------
import play from './ani/play.js';
// -------------------------------------
import on from './evt/on.js';
import off from './evt/off.js';
import trigger from './evt/trigger.js';
// -------------------------------------
/*
import getOffsets from './ui/getOffsets.js';
import getCoords from './ui/getCoords.js';
import getSize from './ui/getSize.js';
import getRect from './ui/getRect.js';
import getRectAtOffsets from './ui/getRectAtOffsets.js';
import getScrollParent from './ui/getScrollParent.js';
import getCanScroll from './ui/getCanScroll.js';
import Displacement from './ui/Displacement.js';
*/
import resizeCallback from './ui/geometry/resizeCallback.js';
// -------------------------------------
import ENV from './ENV.js';
import Reflow from './Reflow.js';
// -------------------------------------

var inited = false;
export function INIT() {
	if (inited) {
		return;
	}
    inited = true;
        
    /**
     * @polyfills
     */

    if (!ENV.window.Element.prototype.matches) {
        ENV.window.Element.prototype.matches = 
        ENV.window.Element.prototype.matchesSelector || 
        ENV.window.Element.prototype.mozMatchesSelector ||
        ENV.window.Element.prototype.msMatchesSelector || 
        ENV.window.Element.prototype.oMatchesSelector || 
        ENV.window.Element.prototype.webkitMatchesSelector ||
        function(s) {
            var matches = (this.document || this.ownerDocument).querySelectorAll(s),
                i = matches.length;
            while (--i >= 0 && matches.item(i) !== this) {}
            return i > -1;            
        };
    }

    ENV.params.vendorPrefix = (function() {
        var styles = ENV.window.getComputedStyle(ENV.window.document.documentElement, '');
        var prefix = (Array.prototype.slice.call(styles).join('').match(/-(moz|webkit|ms)-/) || styles.Olink === '' && ['', 'o'])[1];
        var api = (('WebKit|Moz|Ms|O').match(new RegExp('(' + prefix + ')', 'i')) || [])[1];
        return {prefix:prefix, css:'-' + prefix + '-', api:api,};
    })();

    Reflow._run();
};


/**
 * Create a prototyper.
 *
 * @param object ob
 * @param object params
 * 
 * @return void
 */
export function _extend(target, modules, params = {}) {
    var getReturn = (instance, ret) => {
        if (params.factory) {
            if (ret instanceof Promise) {
                return ret.then(_ret => _ret === instance.el ? instance : _ret);
            }
            return ret === instance.el ? instance : ret;
        }
        return ret;
    };
    _each(modules, (name, fn) => {
        if (name === 'INIT') {
            target[name] = fn;
        } else if (name === 'el' || name === 'select' || name === 'selectAll') {
            target[name] = function(input, ...args) {
                var result = fn(input);
                if (params.factory) {
                    if (name === 'selectAll') {
                        result = _arrFrom(result).map(_el => params.factory(_el, ...args));
                    } else {
                        result = params.factory(result, ...args);
                    }
                }
                return result;
            };
        } else {
            target.prototype[name] = function(...args) {
                return getReturn(this, fn(this.el || ENV.window.document.createElement('div'), ...args));
            };
            // Create short form
            if (name.endsWith('Sync') || name.endsWith('Async')) {
                if (params.syncIsDefault && name.endsWith('Sync')) {
                    name = _beforeLast(name, 'Sync');
                } else if (!params.syncIsDefault && name.endsWith('Async')) {
                    name = _beforeLast(name, 'Async');
                }
                // Add to prototype now
                target.prototype[name] = function(...args) {
                    return getReturn(this, fn(this.el || ENV.window.document.createElement('div'), ...args));
                };
            }
        }
    });
};

/**
 * @exports
 */
export {
    el,
    itemize,
	select,
	selectAll,
	data,
	//------------
	cssSync,
	classSync,
	attrSync,
	htmlSync,
	textSync,
	appendSync,
	prependSync,
	//------------
	cssInline,
	cssStylesheet,
	cssAsync,
	classAsync,
	attrAsync,
	htmlAsync,
	textAsync,
	appendAsync,
	prependAsync,
	cssRendering,
	cssTransaction,
	xBorders,
	yBorders,
	zHeight,
	/*
	//------------
	getOffsets,
	getCoords,
	getSize,
	getRect,
	getRectAtOffsets,
	getScrollParent,
	getCanScroll,
	*/
	//------------
	mutationCallback as onmutated,
	connectedCallback as onconnected,
	disconnectedCallback as ondisconnected,
	attrChangeCallback as onattrchange,
	resizeCallback as onresize,
	//------------
	play,
	//------------
	on,
	off,
	trigger,
};